/*
**
**
** MODULE: server.c
**
**
** PROGRAM: phnbk application
**
**          
**
**
*/      

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <malloc.h>

#include "phnbk.h"

#ifdef WIN32 
#define MAIN_DECL _CRTAPI1

#else
#define MAIN_DECL
#include <dce/rpcexc.h>
#endif

#define IFSPEC phnbk_v1_0_s_ifspec

FILE * filehandle;        /* File handle used for phonebook file */
short previous_operation; /* Keeps track of previous phonebook operation */

int 
MAIN_DECL main
              (
               ac, 
               av
              )
int    ac;
char *av[];
{
    unsigned int                i;
    error_status_t              status;
    unsigned char               *string_binding;
    RPC_BINDING_VECTOR          *bvec;

    /*
    **
    ** Specify TCP/IP as a protocol sequences
    */
    status = RpcServerUseProtseq 
               (
                "ncacn_ip_tcp",
                5,
                NULL
               );

    if (status != RPC_S_OK)
        {
        printf("No available protocol sequences\n");
        exit(EXIT_FAILURE);
        }

    /*
    ** register the server interface
    */
    status = RpcServerRegisterIf 
               (
                IFSPEC,
                NULL,
                NULL
               );

    if (status != RPC_S_OK) 
        {
        printf("Can't register interface \n");
        exit(EXIT_FAILURE);
        }

    /*
    ** find out what binding information is actually available
    */
    status = RpcServerInqBindings 
               (
                &bvec 
               );

    if (status != RPC_S_OK) 
        {
        printf("Can't inquire bindings \n");
        exit(EXIT_FAILURE);
        } 

    /*
    ** register with endpoint mapper
    */
    status = RpcEpRegister 
               ( 
                IFSPEC,
                bvec,
                NULL,
                (unsigned char *)"phnbk endpoint"
               );

    if (status != RPC_S_OK) 
        {
        printf("Can't register endpoint\n");
        exit(EXIT_FAILURE);
        } 

    /*
    ** Get the string bindings and print them
    */
    for (i = 0; i < bvec->Count; i++)
        {
        /*
        ** For each binding, convert it to a
        **    string representation
        */
        status = RpcBindingToStringBinding
                   ( 
                    bvec->BindingH[i],
                    &string_binding
                   );

        if (status != RPC_S_OK) 
            {
            printf("Can't get string binding \n");
            exit(EXIT_FAILURE);
            }

        printf("%s\n", string_binding);
        }

    /*
    ** Open the phonebook file
    */
    filehandle = fopen("phnbk.txt","r");

    /*
    ** Server is all ready to start listening for client
    **    requests...
    */
    status = RpcServerListen
               (
                1,
                2, 
                0
               );

    if (status != RPC_S_OK)
        printf("Error: rpc_server_listen() returned \n");

    return (EXIT_FAILURE);
}
#ifdef WIN32

/*************************************************************************/
/***                MIDL_user_allocate / MIDL_user_free                ***/
/*************************************************************************/

void * __RPC_API
MIDL_user_allocate
           (
            size
           )
size_t size;
{

    unsigned char * ptr;
    ptr = malloc( size );
    return ( (void *)ptr );
    
}

void __RPC_API
MIDL_user_free
           (
            object
           )
void * object;
{
    free (object);
}

#endif


